#!/usr/bin/env startpython
from __future__ import print_function
#########################################################################
#                                                                       # 
#         CREATE TRAININGDATA FROM LINEAR TRANSITS                      # 
#                                                                       # 
#     More information:                                                 # 
#     PLAMS - www.scm.com/doc/Scripting/PLAMS/PLAMS.html                #
#     KF    - www.scm.com/doc/ADF/Appendices/Binary_result_files.html   #
#                                                                       #
#########################################################################


import sys
import os
import glob
import shutil
from scm.plams import *
from rdkit import Chem
import subprocess
from sys import platform as _platform
import re

#
# print help and exit
#
def help_n_exit(message):
    print message
    sys.exit(1)


#----------------------------------------------------------------------#
#                          MAIN                                        #
#----------------------------------------------------------------------#
if __name__ == '__main__':
    #
    # stop if this script is called without
    # the kf-file as argument
    #
    if len(sys.argv) != 2:
        help_n_exit("\n USAGE: " + str(sys.argv[0]) + " [name of kf-file]\n")     
    kf_filename = sys.argv[1]
    #
    # check if the specified file does actually
    # exist
    #
    if not os.path.isfile(kf_filename):
        help_n_exit("\n Error: " + kf_filename + " is not a file.\n")
    #
    # Initialize plams
    #
    init()
    #
    # Initialize the PLAMS KFReader 
    #
    try:
        KFReader = KFFile(kf_filename)
    except:
        exception = sys.exc_info()[0]
        help_n_exit("\n tried to initialize KFFile: " + kf_filename + " ->  Error: %s " % exception + "\n")
    
    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
    #                                                                 #
    #               YOUR CODE BELOW THIS...                           #
    #                                                                 # 
    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
    #
    # create adfreport script
    #
    kf_filename_base = os.path.splitext(kf_filename)[0]
    NumberOfLTPoints = KFReader.read('LT','nr of points')
    ListOfIDs        = []
    Script           = open('get_geometries.sh','w')
    ADFreportScript  = "adfreport " + kf_filename + " -r step\#lt1 -r bgf > geo\n"       
    ListOfIDs.append(kf_filename_base + '_1')
    for LTPoint in range(2,NumberOfLTPoints+1):
        ADFreportScript  += "adfreport " + kf_filename + " -r step\#lt" + str(LTPoint) + " -r bgf >> geo\n"
        ListOfIDs.append(kf_filename_base + '_' + str(LTPoint))
    Script.write(ADFreportScript)
    Script.close()
    #
    # RUN the script
    #
    if _platform == "win32":
        process = subprocess.Popen(['run','sh','get_geometries.sh'])
    else:
        process = subprocess.Popen(['sh','get_geometries.sh'])
    process.wait() 
    #
    # read NumAtoms
    #
    NumAtoms         = KFReader.read('Geometry','nr of atoms')
    #print NumAtoms
    #
    # read Energies
    #
    Energies         = KFReader.read('LT','Energies')
    Energies = [x * 627.50961 for x in Energies] # Hartree 2 kcal/mol 
    #print Energies, min(Energies)
    #
    # read Charges
    #
    Charges          = KFReader.read('LT','AtomCharge Mulliken')
    #print Charges
    #
    # loop through the geo file and change DESCR and ENERGIES
    #
    fout = open('geos_'+kf_filename_base,'w') 
    fin  = open('geo','r')
    counter = 0
    for line in fin: 
        if re.search('DESCRP', line):
            fout.write('DESCRP ' + str(ListOfIDs[counter]) + '\n')
        elif re.search('MOLCHARGE', line):
            fout.write('RUTYPE SINGLE POINT\n')
            fout.write(line)
        elif re.search('ENERGY', line) and not re.search('UNIT', line):
            fout.write('ENERGY ' + str(Energies[counter]) + '\n')
            counter += 1
        else:
            fout.write(line)    
    fin.close()
    fout.close()
    #
    # remove the geo file
    #
    os.remove('geo')
    #
    # write MCFF input
    #
    fout = open('trainingdata_'+kf_filename_base,'w')
    # ENERGIES
    fout.write('ENERGY\n')
    for i in range(0,len(Energies)):
        if Energies[i] == 0.0 or Energies[i] == min(Energies):
            continue
        strbuff  = ' 1.0 + '
        strbuff += ListOfIDs[i] + '/1 - ' 
        strbuff += ListOfIDs[Energies.index(min(Energies))] + '/1 '  
        strbuff += "{:4.4f}\n".format(Energies[i]-min(Energies)) 
        fout.write(strbuff) 
    fout.write('ENDENERGY\n')
    # CHARGES
    fout.write('CHARGE\n')
    strbuff = ' '
    counter = 0
    for Id in ListOfIDs:
        if Energies[counter] == 0.0:
            counter += 1
            continue
        for i in range(0,NumAtoms):
            strbuff += ' ' + Id + ' 0.10 ' + str(i+1) + ' ' 
            strbuff += "{:4.2f}".format(Charges.pop(0)) + '\n'
        fout.write(strbuff)
        strbuff = ''
        counter += 1
            
    fout.write('ENDCHARGE\n')
    fout.close()
    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
    #                                                                 #
    #               YOUR CODE ABOVE THIS...                           #
    #                                                                 # 
    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#    
    #
    # Finish PLAMS
    #        
    finish()
    #
    # remove the plams working directory 
    #
    for name in glob.glob('plams.*'):
        shutil.rmtree(name, ignore_errors=True)
